@@ -26,13 +26,14 @@ from .gammatone import fft_weights
 
 def read_raw_audio(audio, sample_rate=16000):
     if isinstance(audio, str):
-        wave, _ = librosa.load(os.path.expanduser(audio), sr=sample_rate)
+        wave, _ = librosa.load(os.path.expanduser(audio), sr=sample_rate, mono=True)
     elif isinstance(audio, bytes):
         wave, sr = sf.read(io.BytesIO(audio))
+        if wave.ndim > 1: wave = np.mean(wave, axis=-1)
         wave = np.asfortranarray(wave)
-        if sr != sample_rate:
-            wave = librosa.resample(wave, sr, sample_rate)
+        if sr != sample_rate: wave = librosa.resample(wave, sr, sample_rate)
     elif isinstance(audio, np.ndarray):
+        if audio.ndim > 1: ValueError("input audio must be single channel")
         return audio
     else:
         raise ValueError("input audio must be either a path or bytes")
@@ -183,17 +184,17 @@ class SpeechFeaturizer(metaclass=abc.ABCMeta):
         }
         """
         # Samples
-        self.sample_rate = speech_config["sample_rate"]
-        self.frame_length = int(self.sample_rate * (speech_config["frame_ms"] / 1000))
-        self.frame_step = int(self.sample_rate * (speech_config["stride_ms"] / 1000))
+        self.sample_rate = speech_config.get("sample_rate", 16000)
+        self.frame_length = int(self.sample_rate * (speech_config.get("frame_ms", 25) / 1000))
+        self.frame_step = int(self.sample_rate * (speech_config.get("stride_ms", 10) / 1000))
         # Features
-        self.num_feature_bins = speech_config["num_feature_bins"]
-        self.feature_type = speech_config["feature_type"]
-        self.preemphasis = speech_config["preemphasis"]
+        self.num_feature_bins = speech_config.get("num_feature_bins", 80)
+        self.feature_type = speech_config.get("feature_type", "log_mel_spectrogram")
+        self.preemphasis = speech_config.get("preemphasis", None)
         # Normalization
-        self.normalize_signal = speech_config["normalize_signal"]
-        self.normalize_feature = speech_config["normalize_feature"]
-        self.normalize_per_feature = speech_config["normalize_per_feature"]
+        self.normalize_signal = speech_config.get("normalize_signal", True)
+        self.normalize_feature = speech_config.get("normalize_feature", True)
+        self.normalize_per_feature = speech_config.get("normalize_per_feature", False)
 
     @property
     def nfft(self) -> int:
@@ -245,7 +246,7 @@ class NumpySpeechFeaturizer(SpeechFeaturizer):
     def stft(self, signal):
         return np.square(
             np.abs(librosa.core.stft(signal, n_fft=self.nfft, hop_length=self.frame_step,
-                                     win_length=self.frame_length, center=True, window="hann")))
+                                     win_length=self.frame_length, center=False, window="hann")))
 
     def power_to_db(self, S, ref=1.0, amin=1e-10, top_db=80.0):
         return librosa.power_to_db(S, ref=ref, amin=amin, top_db=top_db)
@@ -302,7 +303,7 @@ class NumpySpeechFeaturizer(SpeechFeaturizer):
         pitches, _ = librosa.core.piptrack(
             y=signal, sr=self.sample_rate,
             n_fft=self.nfft, hop_length=self.frame_step,
-            fmin=0.0, fmax=int(self.sample_rate / 2), win_length=self.frame_length, center=True
+            fmin=0.0, fmax=int(self.sample_rate / 2), win_length=self.frame_length, center=False
         )
 
         pitches = pitches.T
